// Copyright (C) 2012 - Prateek Papriwal
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
//

function X = distfun_geoinv(varargin)
    //  Geometric Inverse CDF
    //
    // Calling Sequence
    //   X = distfun_geoinv(p,Pr)
    //   X = distfun_geoinv(p,Pr,lowertail)
    //
    // Parameters
    //   p : a 1x1 or nxm matrix of doubles, the probability .
    //   Pr : a 1x1 or nxm matrix of doubles,  the probability of success in a Bernoulli trial 
    //   lowertail : a 1x1 matrix of booleans, the tail (default lowertail=%t). If lowertail is true (the default), then considers P(X<x) otherwise P(X>x).
    //   X : a nxm matrix of doubles, the outcome. X belongs to the set {0,1,2,3,......}
    //
    // Description
    //   Computes the Inverse cumulative distribution function of 
    //   the Geometric distribution function.
    //   
    //   Any scalar input argument is expanded to a matrix of doubles 
    //   of the same size as the other input arguments.
    //
    // Examples
    //
    // // Test with X scalar, Pr scalar
    //X = distfun_geoinv(0.999,0.5)
    //expected = 9;
    //X = distfun_geoinv(1-0.999,0.5,%f)
    //expected = 9;
    //
    // // Test with expanded X , scalar Pr
    //X = distfun_geoinv([0.32 0.3],0.2)
    //expected = [1. 1.];
    //
    // // Test with scalar X, expanded Pr
    //X = distfun_geoinv(0.22,[0.33 0.1])
    //expected = [0. 2.];
    //
    //
    // // Test small values of p
    // Xn = distfun_geoinv(1.e-15,0.1)
    // expected = 0.;
    // Xn = distfun_geoinv(1.e-15,0.1,%f)
    // expected = 327.;
	//
    // // Test small values of Pr
    // Xn = distfun_geoinv(0.1,1.e-20)
    // expected = 10536051565782630122;
    // Xn = distfun_geoinv(0.1,1.e-20,%f)
    // expected = 230258509299404568400;
    //
    // Bibliography
    // http://en.wikipedia.org/wiki/Geometric_distribution
    //
    // Authors
    // Copyright (C) 2012 - Prateek Papriwal
    //

    [lhs,rhs] = argn()
    apifun_checkrhs("distfun_geoinv",rhs,2:3)
    apifun_checklhs("distfun_geoinv",lhs,0:1)

    p = varargin(1)
    Pr = varargin(2)
    lowertail = apifun_argindefault(varargin,3,%t)

    //
    //Check type
    //
    apifun_checktype("distfun_geoinv",p,"p",1,"constant")
    apifun_checktype("distfun_geoinv",Pr,"Pr",2,"constant")
    apifun_checktype("distfun_geoinv",lowertail,"lowertail",3,"boolean")

    apifun_checkscalar("distfun_geoinv",lowertail,"lowertail",3)
    //
    //Check Content
    apifun_checkrange("distfun_geoinv",p,"p",1,0,1)
    apifun_checkrange("distfun_geoinv",Pr,"Pr",2,0,1)

    [p,Pr] = apifun_expandvar(p,Pr)

    path = distfun_getpath()
    internallib = lib(fullfile(path,"macros","internals"))
    q = distfun_p2q(p)
    if (p==[]) then
        X=[]
        return
    elseif (Pr==[]) then
        X=[]
        return
    elseif (lowertail) then
        X = floor(specfun_log1p(-p) ./ specfun_log1p(-Pr))
    else
        X = floor(specfun_log1p(-q) ./ specfun_log1p(-Pr))
    end
endfunction