// Copyright (C) 2012 - Prateek Papriwal
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt


function y = distfun_geopdf(varargin)
    // Geometric PDF
    //
    // Calling Sequence
    //   y = distfun_geopdf(X,Pr)
    //   
    // Parameters
    //   X : a 1x1 or nxm matrix of doubles, the number of Bernoulli trials after which first success occurs . X belongs to the set {0,1,2,3,.....}
    //   Pr : a 1x1 or nxm matrix of doubles,  the probability of success in a Bernoulli trial 
    //   y : a nxm matrix of doubles, the Probability density.
    //
    // Description
    //   Computes the probability distribution function of 
    //   the Geometric distribution function.
    //   
    //   Any scalar input argument is expanded to a matrix of doubles 
    //   of the same size as the other input arguments.
    //
    //   The function definition is:
    //
    //<latex>
    //\begin{eqnarray}
    //f(X,Pr) = Pr\left(1-Pr\right)^X
    //\end{eqnarray}
    //</latex>
    //
    // Note : X belongs to the set {0,1,2,3,...}. 
    // This choice is compatible with Matlab and R. 
    //
    // Examples
    // // Test X scalar , Pr scalar
    // computed = distfun_geopdf(3,0.5)
    // expected = 0.0625;
    //
    // // Test with X expanded, with Pr scalar
    //computed = distfun_geopdf([2 3],0.1)
    //expected = [0.081 0.0729];
    //
    // //Test with X scalar, Pr expanded
    //computed = distfun_geopdf(3,[0.2 0.4])
    //expected = [0.1024 0.0864];
    //
    // //Test with both arguments expanded
    //computed = distfun_geopdf([3 4 8],[0.5 0.8 0.2])
    //expected = [0.0625 0.00128 0.033554432];
    //
    // // Plot the function
    // // Note that geometric distribution is discrete in nature 
    // // i.e it gives output only at discrete values of X = {0,1,2,3,.....}
    // scf();
    // X = 0:10;
    // y = distfun_geopdf(X,0.2);
    // plot(X,y,"ro-");
    // y1 = distfun_geopdf(X,0.5);
    // plot(X,y1,"go-");
    // y2 = distfun_geopdf(X,0.8);
    // plot(X,y2,"bo-");
    // xtitle("Geometric PDF","x","P(X=x)");
    // legend(["Pr=0.2","Pr=0.5","Pr=0.8"]);
    //
    // Bibliography
    // http://en.wikipedia.org/wiki/Geometric_distribution
    //
    // Authors
    // Copyright (C) 2012 - Prateek Papriwal

    [lhs,rhs] = argn()
    apifun_checkrhs("distfun_geopdf",rhs,2)
    apifun_checklhs("distfun_geopdf",lhs,0:1)

    X=varargin(1)
    Pr=varargin(2)
    //
    // Check type
    //
    apifun_checktype("distfun_geopdf",X,"X",1,"constant")
    apifun_checktype("distfun_geopdf",Pr,"Pr",2,"constant")
    //
    // Check content
    //  
    apifun_checkgreq("distfun_geopdf",X,"X",1,0)
    apifun_checkrange("distfun_geopdf",Pr,"Pr",2,0,1)

    [X,Pr] = apifun_expandvar(X,Pr)

    y = Pr.*((1-Pr).^X)
endfunction