// Copyright (C) 2012 - Prateek Papriwal
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt


function R = distfun_geornd(varargin)
    // Geometric random numbers
    //
    // Calling Sequence
    //   R = distfun_geornd(Pr)
    //   R = distfun_geornd(Pr,v)
    //   R = distfun_geornd(Pr,m,n)
    //
    // Parameters
    //   Pr : a 1x1 or nxm matrix of doubles, the probability of getting success in a Bernoulli trial
    //   v : a 1x2 or 2x1 matrix of doubles, the size of R
    //   v(1) : the number of rows of R
    //   v(2) : the number of columns of R
    //   m : a 1x1 matrix of floating point integers, the number of rows of R
    //   n : a 1x1 matrix of floating point integers, the number of columns of R
    //   R: a matrix of doubles, the random numbers.
    //
    // Description
    //   Generates random variables from the Geometric distribution function.
    //
    //   Any scalar input argument is expanded to a matrix of doubles 
    //   of the same size as the other input arguments.
    //
    //   As a side effect, it modifies the internal seed of the grand function.
    //
    // Note - The output argument R belongs to the set {0,1,2,3,...}. 
    // This is not compatible with the <literal>grand(m,n,"geom",p)</literal> function in Scilab v5, 
    // where the choice is the set {1,2,3,...}. 
    // In other words, the calling sequence 
    //
    //<screen>
    // R = grand(m,n,"geom",Pr)
    //</screen>
    //
    // is equivalent to 
    //
    //<screen>
    // R = distfun_geornd(Pr,m,n) + 1
    //</screen>
    //
    // Examples
    // // set the initial seed for tests
    // distfun_seedset(1);
    // // Test with expanded Pr
    // computed = distfun_geornd(1 ./(1:6))
    // expected = [1 0 14 4 12 0];
    //
    // // Check expansion of Pr in R = distfun_geornd(Pr)
    // distfun_seedset(1);
    // N = 10;
    // computed1(1:6,i) = distfun_geornd(1 ./(1:6))
    //
    // // Check R = distfun_geornd(Pr,v)
    // computed = distfun_geornd(0.2,[4 5])
    // assert_checkequal(size(computed),[4 5]);
    //
    // // Check mean and variance
    // N = 5000;
    // Pr = 0.3;
    // computed = distfun_geornd(Pr,[1 N]);
    // c = mean(computed(1:N))
    // d = st_deviation(computed(1:N) )
    // [M,V] = distfun_geostat (Pr)
    //
    // Bibliography
    // http://en.wikipedia.org/wiki/Geometric_distribution
    // 
    // Authors
    // Copyright (C) 2012 - Prateek Papriwal
    //

    path = distfun_getpath()
    internallib = lib(fullfile(path,"macros","internals"))

    [lhs,rhs] = argn()

    apifun_checkrhs("distfun_geornd",rhs,1:3)
    apifun_checklhs("distfun_geornd",lhs,0:1)

    Pr = varargin(1)
    //
    // Check type
    //
    apifun_checktype("distfun_geornd",Pr,"Pr",1,"constant")

    if ( rhs == 2 ) then
        v = varargin(2)
    end
    if ( rhs == 3 ) then
        m = varargin(2)
        n = varargin(3)
    end

    //
    // Check v,m,n
    //
    distfun_checkvmn ( "distfun_geornd" , 2 , varargin(2:$) )

    Pr = apifun_expandfromsize ( 1 , varargin(1:$) )

    if(Pr == []) then
        R = []
        return
    end

    m = size(Pr,"r")
    n = size(Pr,"c")

    R = distfun_grandgeom(m,n,Pr)

endfunction
