// Copyright (C) 2012 - Prateek Papriwal
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

function [M,V] = distfun_geostat (Pr)
    // Geometric  mean and variance
    //
    // Calling Sequence
    //   M = distfun_geostat(Pr)
    //   [M,V] = distfun_unifstat(Pr)
    //
    // Parameters
    //   Pr : a 1x1 or nxm matrix of doubles, the probability of getting success in a Bernoulli trial 
    //   M : a matrix of doubles, the mean
    //   V : a matrix of doubles, the variance
    //
    // Description
    //   Computes statistics from the Geometric distribution.
    //
    //   Any scalar input argument is expanded to a matrix of 
    //   doubles of the same size as the other input arguments.
    //
    // The Mean and Variance of the Geometric Distribution are
    //
    //<latex>
    //\begin{eqnarray}
    // M &=& \frac{1-Pr}{Pr} \\
    // V &=& \frac{1-Pr}{Pr^2}
    //\end{eqnarray}
    //</latex>
    // 
    // Examples
    // // Test with expanded Pr
    //[m,v] = distfun_geostat(1 ./(1:6))
    //me = [ 0  1.0000  2.0000  3.0000  4.0000  5.0000];
    //ve = [ 0  2.0000  6.0000  12.0000  20.0000  30.0000];
    //
    // //Accuracy test
    //Pr=[0.11 0.22 0.33];
    //[M,V] = distfun_geostat ( Pr )
    //ve= [73.553719 16.115702 6.1524334 ];
    //me = [8.0909091 3.5454545 2.030303 ];
    //
    // Bibliography
    //http://en.wikipedia.org/wiki/Geometric_distribution
    // 
    // Authors
    // Copyright (C) 2012 - Prateek Papriwal
    //

    [lhs,rhs] = argn()
    apifun_checkrhs("distfun_geostat",rhs,1)
    apifun_checklhs("distfun_geostat",lhs,1:2)
    // Check type
    //
    apifun_checktype("distfun_geostat",Pr,"Pr",1,"constant")

    //Check content
    // 
    apifun_checkrange("distfun_geostat",Pr,"Pr",1,0,1)

    //
    [Pr] = apifun_expandvar(Pr)

    M = (1-Pr) ./Pr
    V = (1-Pr) ./ Pr.^2
endfunction