// =============================================================================

// Copyright (C) 2012 - Michael Baudin
// Copyright (C) 2008 - INRIA - Sabine Gaüzere
// Copyright (C) 2010-2011 - DIGITEO - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt
// =============================================================================

//
// A set of utilities to test distribution functions.
//

function [x,y] = histcompute(n,data)
    //
    // Computes the histogram of a data.
    //
    // Parameters
    // n : a 1-by-1 matrix of floating point integers, the number of classes.
    // data : a matrix of doubles, the data
    // x : 1-by-n+1 matrix of doubles, the classes of the histogram
    // y : 1-by-n+1 matrix of doubles, the empirical probability that one value in data which fall in each class
    x = linspace(min(data), max(data), n+1);
    [ind , y] = dsearch(data, x)
    y = y./length(data)
endfunction

function checkLaw0arg ( R , cdffun , N , NC , rtol , atol )
    //
    // Check the random number generator for a continuous distribution function.
    //
    // Parameters
    // R: a N-by-1 matrix, the random numbers
    // cdffun : a function, the Cumulated Distribution Function
    // NC : a 1-by-1 matrix of floating point integers, the number of classes
    // N : a 1-by-1 matrix of floating point integers, the number of random values to test
    // rtol : a 1-by-1 matrix of doubles, the relative tolerance
    //
    // Description
    //  Compare the empirical histogram with the theoretical PDF.
    //  Compare the empirical CDF with the theoretical CDF.
    //  The cdffun function must have the header 
    //
    //  P = cdffun(X)
    //

    R = R(:);
    // Compare the empirical histogram with the PDF
    [X,EmpiricalPDF] = histcompute(NC,R);
    CDF = cdffun(X);
    TheoricPDF = diff(CDF);
    assert_checkalmostequal( EmpiricalPDF,TheoricPDF, rtol ,  atol , "element" );
    if ( %f ) then
        scf();
        plot(X(1:$-1),EmpiricalPDF,"bo-"); // Empirical Histogram
        plot(X(1:$-1),TheoricPDF,"rox-"); // Theoretical Histogram
        legend(["Empirical PDF","Theoretical PDF"]);
    end
    //
    // Compare the Empirical CDF and the Theoretical CDF
    RdevS=gsort(R,"g","i");
    PS=(1:N)'/N;
    P=cdffun(RdevS);
    assert_checkalmostequal( P, PS , rtol ,  atol , "element" );
    if ( %f ) then
        scf();
        plot(RdevS,PS,"b-"); // Empirical distribution
        plot(RdevS,P,"r-"); // Theoretical distribution
        legend(["Empirical CDF","Theoretical CDF"]);
		// P-P Plot
        scf();
        plot(PS,PS,"b-"); // Straight Line
        plot(PS,P,"r-"); // Data
        legend(["Theory","Data"]);
    end

endfunction

function checkLaw1arg ( R , cdffun , N , NC , A , rtol , atol )
    //
    // Check the random number generator for a continuous distribution function.
    //
    // Parameters
    // R: a N-by-1 matrix, the random numbers
    // cdffun : a function, the Cumulated Distribution Function
    // NC : a 1-by-1 matrix of floating point integers, the number of classes
    // N : a 1-by-1 matrix of floating point integers, the number of random values to test
    // A : a 1-by-1 matrix of doubles, the value of the parameter
    // rtol : a 1-by-1 matrix of doubles, the relative tolerance
    //
    // Description
    //  Compare the empirical histogram with the theoretical PDF.
    //  Compare the empirical CDF with the theoretical CDF.
    //  The cdffun function must have the header 
    //
    //  P = cdffun(X,A)
    //

    R = R(:);
    [X,EmpiricalPDF] = histcompute(NC,R);
    CDF = cdffun(X,A);
    TheoricPDF = diff(CDF);
    assert_checkalmostequal( EmpiricalPDF , TheoricPDF , rtol ,  atol , "element" );
    if ( %f ) then
        scf();
        plot(X(1:$-1),EmpiricalPDF,"bo-"); // Empirical Histogram
        plot(X(1:$-1),TheoricPDF,"rox-"); // Theoretical Histogram
        legend(["Empirical PDF","Theoretical PDF"]);
    end
    //
    // Compare the Empirical CDF and the Theoretical CDF
    RdevS=gsort(R,"g","i");
    PS=(1:N)'/N;
    P=cdffun(RdevS,A);
    assert_checkalmostequal( P, PS , rtol ,  atol , "element" );
    if ( %f ) then
        scf();
        plot(RdevS,PS,"b-"); // Empirical distribution
        plot(RdevS,P,"r-"); // Theoretical distribution
        legend(["Empirical CDF","Theoretical CDF"]);
		// P-P Plot
        scf();
        plot(PS,PS,"b-"); // Straight Line
        plot(PS,P,"r-"); // Data
        legend(["Theory","Data"]);
    end

endfunction

function checkLaw2arg ( R , cdffun , N , NC , A , B , rtol , atol )
    //
    // Check the random number generator for a continuous distribution function.
    //
    // Parameters
    // R: a N-by-1 matrix, the random numbers
    // cdffun : a function, the Cumulated Distribution Function
    // NC : a 1-by-1 matrix of floating point integers, the number of classes
    // N : a 1-by-1 matrix of floating point integers, the number of random values to test
    // A : a 1-by-1 matrix of doubles, the value of the 1st parameter
    // B : a 1-by-1 matrix of doubles, the value of the 2nd parameter
    // rtol : a 1-by-1 matrix of doubles, the relative tolerance
    //
    // Description
    //  Compare the empirical histogram with the theoretical PDF.
    //  Compare the empirical CDF with the theoretical CDF.
    //  The cdffun function must have the header 
    //
    //  P = cdffun(X,A,B)
    //

    R = R(:);
    [X,EmpiricalPDF] = histcompute(NC,R);
    CDF = cdffun(X,A,B);
    TheoricPDF = diff(CDF);
    assert_checkalmostequal( EmpiricalPDF , TheoricPDF , rtol , atol , "element" );
    if ( %f ) then
        scf();
        plot(X(1:$-1),EmpiricalPDF,"bo-"); // Empirical Histogram
        plot(X(1:$-1),TheoricPDF,"rox-"); // Theoretical Histogram
        legend(["Empirical PDF","Theoretical PDF"]);
    end
    //
    // Compare the Empirical CDF and the Theoretical CDF
    RdevS=gsort(R,"g","i");
    PS=(1:N)'/N;
    P=cdffun(RdevS,A,B);
    assert_checkalmostequal( P, PS , rtol , atol , "element" );
    if ( %f ) then
        scf();
        plot(RdevS,PS,"b-"); // Empirical distribution
        plot(RdevS,P,"r-"); // Theoretical distribution
        legend(["Empirical CDF","Theoretical CDF"]);
		// P-P Plot
        scf();
        plot(PS,PS,"b-"); // Straight Line
        plot(PS,P,"r-"); // Data
        legend(["Theory","Data"]);
    end
endfunction

function checkLaw3arg ( R , cdffun , N , NC , A , B , C , rtol , atol )
    //
    // Check the random number generator for a continuous distribution function.
    //
    // Parameters
    // R: a 1-by-N matrix, the random numbers
    // cdffun : a function, the Cumulated Distribution Function
    // NC : a 1-by-1 matrix of floating point integers, the number of classes
    // N : a 1-by-1 matrix of floating point integers, the number of random values to test
    // A : a 1-by-1 matrix of doubles, the value of the 1st parameter
    // B : a 1-by-1 matrix of doubles, the value of the 2nd parameter
    // C : a 1-by-1 matrix of doubles, the value of the 3d parameter
    // rtol : a 1-by-1 matrix of doubles, the relative tolerance
    //
    // Description
    //  Compare the empirical histogram with the theoretical histogram.
    //  The cdffun function must have the header 
    //
    //  P = cdffun(X,A,B,C)
    //

    R = R(:);
    [X,EmpiricalPDF] = histcompute(NC,R);
    CDF = cdffun(X,A,B,C)
    TheoricPDF = diff(CDF);
    assert_checkalmostequal( EmpiricalPDF , TheoricPDF , rtol ,  atol , "element" );
    if ( %f ) then
        scf();
        plot(X(1:$-1),EmpiricalPDF,"bo-"); // Empirical Histogram
        plot(X(1:$-1),TheoricPDF,"rox-"); // Theoretical Histogram
        legend(["Empirical PDF","Theoretical PDF"]);
    end
    //
    // Compare the Empirical CDF and the Theoretical CDF
    RdevS=gsort(R,"g","i");
    PS=(1:N)'/N;
    P=cdffun(RdevS,A,B,C);
    assert_checkalmostequal( P, PS , rtol , atol , "element" );
    if ( %f ) then
        scf();
        plot(RdevS,PS,"b-"); // Empirical distribution
        plot(RdevS,P,"r-"); // Theoretical distribution
        legend(["Empirical CDF","Theoretical CDF"]);
    end
endfunction

function checkPieceLaw0arg ( R , cdffun , N , NC , rtol , atol )
    //
    // Check the random number generator for a piecewise distribution function.
    //
    // Parameters
    // R: a N-by-1 matrix of doubles
    // cdffun : a function, the Cumulated Distribution Function
    // NC : a 1-by-1 matrix of floating point integers, the number of classes
    // N : a 1-by-1 matrix of floating point integers, the number of random values to test
    // rtol : a 1-by-1 matrix of doubles, the relative tolerance
    //
    // Description
    //  Compare the empirical histogram with the theoretical histogram.
    //  The classes of the histogram are computed from the random samples, 
    //  from which the unique entries are extracted.

    R = R(:);
    X = unique(R);
    for k = 1 : size(X,"*")
        EmpiricalPDF(k) = length(find(R==X(k)));
    end
    EmpiricalPDF = EmpiricalPDF./N;
    CDF = cdffun(X);
    TheoricPDF=[CDF(1);diff(CDF)];
    assert_checkalmostequal( EmpiricalPDF , TheoricPDF , rtol ,  atol , "element" );
    if ( %f ) then
        scf();
        plot(X,EmpiricalPDF,"bo-"); // Empirical Histogram
        plot(X,TheoricPDF,"rox-"); // Theoretical Histogram
        legend(["Empirical PDF","Theoretical PDF"]);
    end
    //
    // Compare the Empirical CDF and the Theoretical CDF
    EmpiricalCDF = cumsum(EmpiricalPDF);
    P=cdffun(X);
    assert_checkalmostequal( P, EmpiricalCDF , rtol ,  atol , "element" );
    if ( %f ) then
        scf();
        plot(X,EmpiricalCDF,"b-"); // Empirical distribution
        plot(X,P,"r-"); // Theoretical distribution
        legend(["Empirical CDF","Theoretical CDF"]);
    end
endfunction

function checkPieceLaw1arg ( R , cdffun , N , NC , A , rtol , atol )
    //
    // Check the random number generator for a piecewise distribution function.
    //
    // Parameters
    // R: a N-by-1 matrix, the random numbers
    // cdffun : a function, the Cumulated Distribution Function
    // NC : a 1-by-1 matrix of floating point integers, the number of classes
    // N : a 1-by-1 matrix of floating point integers, the number of random values to test
    // A : a 1-by-1 matrix of doubles, the value of the parameter
    // rtol : a 1-by-1 matrix of doubles, the relative tolerance
    //
    // Description
    //  Compare the empirical histogram with the theoretical histogram.
    //  The classes of the histogram are computed from the random samples, 
    //  from which the unique entries are extracted.

    R = R(:);
    X = unique(R);
    for k = 1 : size(X,"*")
        EmpiricalPDF(k) = length(find(R==X(k)));
    end
    EmpiricalPDF = EmpiricalPDF./N;
    CDF = cdffun(X,A);
    TheoricPDF=[CDF(1);diff(CDF)];
    assert_checkalmostequal( EmpiricalPDF  , TheoricPDF , rtol ,  atol , "element" );
    if ( %f ) then
        scf();
        plot(X,EmpiricalPDF,"bo-"); // Empirical Histogram
        plot(X,TheoricPDF,"rox-"); // Theoretical Histogram
        legend(["Empirical PDF","Theoretical PDF"]);
    end
    //
    // Compare the Empirical CDF and the Theoretical CDF
    EmpiricalCDF = cumsum(EmpiricalPDF);
    P=cdffun(X,A);
    assert_checkalmostequal( P, EmpiricalCDF , rtol ,  atol , "element" );
    if ( %f ) then
        scf();
        plot(X,EmpiricalCDF,"b-"); // Empirical distribution
        plot(X,P,"r-"); // Theoretical distribution
        legend(["Empirical CDF","Theoretical CDF"]);
    end
endfunction

function checkPieceLaw2arg ( R , cdffun , N , NC , A , B , rtol , atol )
    //
    // Check the random number generator for a piecewise distribution function.
    //
    // Parameters
    // R: a N-by-1 matrix, the random numbers
    // cdffun : a function, the Cumulated Distribution Function
    // NC : a 1-by-1 matrix of floating point integers, the number of classes
    // N : a 1-by-1 matrix of floating point integers, the number of random values to test
    // A : a 1-by-1 matrix of doubles, the value of the parameter
    // rtol : a 1-by-1 matrix of doubles, the relative tolerance
    //
    // Description
    //  Compare the empirical histogram with the theoretical histogram.
    //  The classes of the histogram are computed from the random samples, 
    //  from which the unique entries are extracted.

    R = R(:);
    X = unique(R);
    for k = 1 : size(X,"*")
        EmpiricalPDF(k) = length(find(R==X(k)));
    end
    EmpiricalPDF = EmpiricalPDF./N;
    CDF = cdffun(X,A,B);
    TheoricPDF=[CDF(1);diff(CDF)];
    assert_checkalmostequal( EmpiricalPDF , TheoricPDF ,  rtol ,  atol , "element" );
    if ( %f ) then
        scf();
        plot(X,EmpiricalPDF,"bo-"); // Empirical Histogram
        plot(X,TheoricPDF,"rox-"); // Theoretical Histogram
        legend(["Empirical PDF","Theoretical PDF"]);
    end
    //
    // Compare the Empirical CDF and the Theoretical CDF
    EmpiricalCDF = cumsum(EmpiricalPDF);
    P=cdffun(X,A,B);
    assert_checkalmostequal( P, EmpiricalCDF , rtol ,  atol , "element" );
    if ( %f ) then
        scf();
        plot(X,EmpiricalCDF,"b-"); // Empirical distribution
        plot(X,P,"r-"); // Theoretical distribution
        legend(["Empirical CDF","Theoretical CDF"]);
    end
endfunction

function checkMeanVariance ( m , n , R , mu , va , rtol )
    // Check the mean and variance of random numbers.
    //
    // Parameters
    // m : a 1-by-1 matrix of floating point integers, the number of rows
    // n : a 1-by-1 matrix of floating point integers, the number of columns
    // R: a m-by-n matrix, the random numbers
    // mu : a 1-by-1 matrix of doubles, the expected mean
    // va : a 1-by-1 matrix of doubles, the expected variance. If va==[], then the variance is not checked.
    // rtol : a 1-by-1 matrix of doubles, the relative tolerance

    assert_checkequal ( size(R) , [m,n] );
    assert_checkequal ( typeof(R) , "constant" );
    assert_checkalmostequal ( mean(R) , mu , rtol );
    if ( va<>[] ) then
        assert_checkalmostequal ( variance(R) , va , rtol );
    end
endfunction

function checkExpansion1arg ( m , n , mygrandfun , a)
    // Check that a argument of a distfun_grand*(m,n,a) function can be expanded.
    //
    // Parameters
    // m : a 1-by-1 matrix of floating point integers, the number of rows
    // n : a 1-by-1 matrix of floating point integers, the number of columns
    // mygrandfun : a function, the distfun_grand* function
    // a : a m-by-n matrix of floating point integers, the first argument
    //
    // mygrandfun must have the header : R = mygrandfun(m,n,a,b)

    R=mygrandfun(m,n,a);
    assert_checkequal ( size(R) , [m,n] );
    //
	// Check expansion of a
    distfun_seedset(0);
    R1 = mygrandfun(m,n,a(1));
    assert_checkequal ( size(R1) , [m,n] );
    distfun_seedset(0);
    R2 = mygrandfun(m,n,a(1)*ones(m,n));
    assert_checkequal ( R1 , R2 );
endfunction

function checkExpansion2args ( m , n , mygrandfun , a, b)
    // Check that (a,b) arguments of a distfun_grand*(m,n,a,b) function can be expanded.
    //
    // Parameters
    // m : a 1-by-1 matrix of floating point integers, the number of rows
    // n : a 1-by-1 matrix of floating point integers, the number of columns
    // mygrandfun : a function, the distfun_grand* function
    // a : a m-by-n matrix of floating point integers, the first argument
    // b : a m-by-n matrix of floating point integers, the second argument
    //
    // mygrandfun must have the header : R = mygrandfun(m,n,a,b)

    R=mygrandfun(m,n,a,b);
    assert_checkequal ( size(R) , [m,n] );
    //
	// Check expansion of a and b
    distfun_seedset(0);
    R1 = mygrandfun(m,n,a(1),b(1));
    assert_checkequal ( size(R1) , [m,n] );
    distfun_seedset(0);
    R2 = mygrandfun(m,n,a(1)*ones(m,n),b(1)*ones(m,n));
    assert_checkequal ( R1 , R2 );
endfunction

function checkExpansion3args ( m , n , mygrandfun , a, b, c)
    // Check that (a,b,c) arguments of a distfun_grand*(m,n,a,b,c) function can be expanded.
    //
    // Parameters
    // m : a 1-by-1 matrix of floating point integers, the number of rows
    // n : a 1-by-1 matrix of floating point integers, the number of columns
    // mygrandfun : a function, the distfun_grand* function
    // a : a m-by-n matrix of floating point integers, the first argument
    // b : a m-by-n matrix of floating point integers, the second argument
    // c : a m-by-n matrix of floating point integers, the third argument
    //
    // mygrandfun must have the header : R = mygrandfun(m,n,a,b,c)

    R=mygrandfun(m,n,a,b,c);
    assert_checkequal ( size(R) , [m,n] );
    //
	// Check expansion of a and b and c
    distfun_seedset(0);
    R1 = mygrandfun(m,n,a(1),b(1),c(1));
    assert_checkequal ( size(R1) , [m,n] );
    distfun_seedset(0);
    R2 = mygrandfun(m,n,a(1)*ones(m,n),b(1)*ones(m,n),c(1)*ones(m,n));
    assert_checkequal ( R1 , R2 );
endfunction

function checkUniformity(kgen,rtol,atol,N,NC,UinMax)
    // Check the uniformity of numbers computed from the RNG
    // Parameters
    // kgen : a 1-by-1 matrix of doubles, integer value, the index of the RNG. In the set 1, 2, ..., NGen
    // rtol : a 1-by-1 matrix of doubles, the relative tolerance
    // N  : a 1-by-1 matrix of doubles, integer value, the number of values to generate
	// UinMax : a 1-by-1 matrix of doubles, integer value, the maximum integer generable with uin option
    //
    // Description
    // Checks the output of the functions 
    // * distfun_grandunf
    // * distfun_granduin 
    //

    //
    // Check distribution of uniform numbers in [0,1[
    R = distfun_grandunf(N,1,0,1);
    checkMeanVariance (N , 1 , R , 1/2 , 1/12 , rtol );
    checkLaw0arg ( R , mycdfdef , N , NC , rtol , atol );
    //
    // Check distribution of uniform integers in [A,B]
    R = distfun_granduin(N,1,0,UinMax);
    checkPieceLaw2arg ( R , mycdfuin , N , NC , 0 , UinMax , rtol );
endfunction

